(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@angular/core'), require('@angular/fire/storage'), require('ngx-image-cropper'), require('@angular/common')) :
    typeof define === 'function' && define.amd ? define('file-upload-firestorage', ['exports', '@angular/core', '@angular/fire/storage', 'ngx-image-cropper', '@angular/common'], factory) :
    (global = global || self, factory(global['file-upload-firestorage'] = {}, global.ng.core, global.ng.fire.storage, global.ngxImageCropper, global.ng.common));
}(this, (function (exports, i0, storage, ngxImageCropper, common) { 'use strict';

    var FILE_SIZE_UNITS = ['B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
    var FILE_SIZE_UNITS_LONG = ['Bytes', 'Kilobytes', 'Megabytes', 'Gigabytes', 'Pettabytes', 'Exabytes', 'Zettabytes', 'Yottabytes'];
    var FileSizePipe = /** @class */ (function () {
        function FileSizePipe() {
        }
        FileSizePipe.prototype.transform = function (sizeInBytes, longForm) {
            var units = longForm
                ? FILE_SIZE_UNITS_LONG
                : FILE_SIZE_UNITS;
            var power = Math.round(Math.log(sizeInBytes) / Math.log(1024));
            power = Math.min(power, units.length - 1);
            var size = sizeInBytes / Math.pow(1024, power); // size in new units
            var formattedSize = Math.round(size * 100) / 100; // keep up to 2 decimals
            var unit = units[power];
            return size ? formattedSize + " " + unit : '0';
        };
        return FileSizePipe;
    }());
    FileSizePipe.decorators = [
        { type: i0.Pipe, args: [{
                    name: 'fileSize'
                },] }
    ];

    var DropZoneDirective = /** @class */ (function () {
        function DropZoneDirective() {
            this.dropped = new i0.EventEmitter();
            this.hovered = new i0.EventEmitter();
        }
        DropZoneDirective.prototype.onDrop = function ($event) {
            console.log($event);
            $event.preventDefault();
            this.dropped.emit($event.dataTransfer.files);
            this.hovered.emit(false);
        };
        DropZoneDirective.prototype.onDragOver = function ($event) {
            $event.preventDefault();
            this.hovered.emit(true);
        };
        DropZoneDirective.prototype.onDragLeave = function ($event) {
            $event.preventDefault();
            this.hovered.emit(false);
        };
        return DropZoneDirective;
    }());
    DropZoneDirective.decorators = [
        { type: i0.Directive, args: [{
                    selector: '[dropZone]'
                },] }
    ];
    DropZoneDirective.ctorParameters = function () { return []; };
    DropZoneDirective.propDecorators = {
        dropped: [{ type: i0.Output }],
        hovered: [{ type: i0.Output }],
        onDrop: [{ type: i0.HostListener, args: ['drop', ['$event'],] }],
        onDragOver: [{ type: i0.HostListener, args: ['dragover', ['$event'],] }],
        onDragLeave: [{ type: i0.HostListener, args: ['dragleave', ['$event'],] }]
    };

    var FileUploadFirestorageService = /** @class */ (function () {
        function FileUploadFirestorageService() {
        }
        return FileUploadFirestorageService;
    }());
    FileUploadFirestorageService.ɵprov = i0.ɵɵdefineInjectable({ factory: function FileUploadFirestorageService_Factory() { return new FileUploadFirestorageService(); }, token: FileUploadFirestorageService, providedIn: "root" });
    FileUploadFirestorageService.decorators = [
        { type: i0.Injectable, args: [{
                    providedIn: 'root'
                },] }
    ];
    FileUploadFirestorageService.ctorParameters = function () { return []; };

    var DropZoneText = /** @class */ (function () {
        function DropZoneText(lang) {
            this.title = 'File Drop Zone';
            this.subtitle = 'Drop your file here';
            this.action = {
                or: 'or',
                click: 'click here',
                choose: 'to choose a file...',
                rotateLeft: 'Rotate left',
                rotateRight: 'Rotate right',
                flipHorizontal: 'Flip horizontal',
                flipVertical: 'Flip Vertical',
                zoomIn: 'Zoom +',
                zoomOut: 'Zoom -',
                reset: 'Reset image',
            };
            this.button = {
                pause: 'Pause',
                cancel: 'Cancel',
                resume: 'Resume',
                upload: 'Upload File',
                change: 'Change File'
            };
            this.errors = {
                type: 'Unsupported file type. ',
                size: 'The file is too heavy. ',
            };
            switch (lang) {
                case 'es':
                    this.title = 'Area de carga';
                    this.subtitle = 'Arroje su archivo acá',
                        this.action = {
                            or: 'ó',
                            click: 'Haga click aquí',
                            choose: 'para escoger su archivo...',
                            rotateLeft: 'Rotar Izq',
                            rotateRight: 'Rotar Der',
                            flipHorizontal: 'Girar horizontal',
                            flipVertical: 'Girar Vertical',
                            zoomIn: 'Acercar',
                            zoomOut: 'Alejar',
                            reset: 'Resetear',
                        };
                    this.button = {
                        pause: 'Pausa',
                        cancel: 'Cancelar',
                        resume: 'Continuar',
                        upload: 'Cargar archivo',
                        change: 'Cambiar archivo'
                    };
                    this.errors = {
                        type: 'El tipo de archivo no es permitido.',
                        size: 'El archivo es muy grande. ',
                    };
                    break;
                default:
                    break;
            }
        }
        return DropZoneText;
    }());

    var FileUploadFirestorageComponent = /** @class */ (function () {
        function FileUploadFirestorageComponent(storage) {
            this.storage = storage;
            this.res = new i0.EventEmitter();
            this.lang = 'en';
            this.cropper = false;
            this.cropperConfig = {
                aspectRatio: 1 / 1,
                maintainAspectRatio: true,
                containWithinAspectRatio: false,
                cropperMinHeight: 0,
                cropperMinWidth: 0,
                resizeToWidth: 0,
                resizeToHeight: 0,
                cropperStaticWidth: 0,
                cropperStaticHeight: 0,
            };
            this.error = {
                type: false,
                size: false
            };
            // Cropped
            this.croppedImage = '';
            this.croppedImageConfig = {
                rotation: 0,
                canvasRotation: 0,
                showCropper: false,
                scale: 1,
                transform: {},
            };
        }
        FileUploadFirestorageComponent.prototype.toggleHover = function (event) {
            this.isHovering = event;
        };
        FileUploadFirestorageComponent.prototype.loadTempFile = function (event) {
            this.error.type = false;
            this.error.size = false;
            this.ImageType = true;
            var file = event.target ? event.target.files.item(0) : event.item(0);
            // Client-side validation example
            if (this.onlyType) {
                var allow = false;
                allow = this.onlyType.split(',').some(function (type) { return file.type.includes(type.trim()); });
                if (!allow) {
                    this.error.type = true;
                    console.error('unsupported file type: ', file.type);
                    return;
                }
            }
            if (this.maxSize && (file.size / 100000) >= this.maxSize) {
                this.error.size = true;
                console.error('unsupported size file. Its too heavy ', file.size);
                return;
            }
            if (!this.error.type && !this.error.size) {
                if (event.target) {
                    this.tempFile = event;
                }
                else {
                    var aux = {
                        target: {
                            files: []
                        }
                    };
                    aux.target.files.push(file);
                    this.tempFile = aux;
                }
                if (file.type.includes('image')) {
                    this.imageLoaded();
                }
                else {
                    this.ImageType = false;
                }
            }
        };
        FileUploadFirestorageComponent.prototype.changeImage = function () {
            this.tempFile = null;
        };
        // startUpload(event: FileList) {
        FileUploadFirestorageComponent.prototype.startUpload = function (event) {
            var _this = this;
            var file;
            if (!this.cropper && this.tempFile) {
                file = this.tempFile.target ? this.tempFile.target.files.item(0) : this.tempFile;
            }
            else {
                file = ngxImageCropper.base64ToFile(this.imageCropper.crop().base64);
            }
            // The File object
            // The storage path
            var path = this.storagePath + "/" + new Date().getTime() + "_" + (this.tempFile ? this.tempFile.target.files[0].name : '_web');
            // Totally optional metadata
            // The main task
            this.task = this.storage.upload(path, file);
            // Progress monitoring
            this.percentage = this.task.percentageChanges();
            this.snapshot = this.task.snapshotChanges();
            // The file's download URL
            this.task.then(function (d) {
                d.ref.getDownloadURL().then(function (url) {
                    _this.snapshot = null;
                    _this.percentage = null;
                    _this.res.emit({ thumbnail: url, locationPath: d.ref.fullPath.split('/').pop() });
                });
            });
        };
        // Determines if the upload task is active
        FileUploadFirestorageComponent.prototype.isActive = function (snapshot) {
            return snapshot.state === 'running' && snapshot.bytesTransferred < snapshot.totalBytes;
        };
        FileUploadFirestorageComponent.prototype.ngOnInit = function () {
            this.texts = new DropZoneText(this.lang);
        };
        // CropImageFunctions 
        FileUploadFirestorageComponent.prototype.rotateLeft = function () {
            this.croppedImageConfig.canvasRotation--;
            this.flipAfterRotate();
        };
        FileUploadFirestorageComponent.prototype.rotateRight = function () {
            this.croppedImageConfig.canvasRotation++;
            this.flipAfterRotate();
        };
        FileUploadFirestorageComponent.prototype.flipAfterRotate = function () {
            var flippedH = this.croppedImageConfig.transform.flipH;
            var flippedV = this.croppedImageConfig.transform.flipV;
            this.croppedImageConfig.transform = Object.assign(Object.assign({}, this.croppedImageConfig.transform), { flipH: flippedV, flipV: flippedH });
        };
        FileUploadFirestorageComponent.prototype.flipHorizontal = function () {
            this.croppedImageConfig.transform = Object.assign(Object.assign({}, this.croppedImageConfig.transform), { flipH: !this.croppedImageConfig.transform.flipH });
        };
        FileUploadFirestorageComponent.prototype.flipVertical = function () {
            this.croppedImageConfig.transform = Object.assign(Object.assign({}, this.croppedImageConfig.transform), { flipV: !this.croppedImageConfig.transform.flipV });
        };
        FileUploadFirestorageComponent.prototype.resetImage = function () {
            this.croppedImageConfig.scale = 1;
            this.croppedImageConfig.transform = {};
            this.croppedImageConfig.rotation = 0;
            this.croppedImageConfig.canvasRotation = 0;
        };
        FileUploadFirestorageComponent.prototype.zoomOut = function () {
            this.croppedImageConfig.scale -= .1;
            this.croppedImageConfig.transform = Object.assign(Object.assign({}, this.croppedImageConfig.transform), { scale: this.croppedImageConfig.scale });
        };
        FileUploadFirestorageComponent.prototype.zoomIn = function () {
            this.croppedImageConfig.scale += .1;
            this.croppedImageConfig.transform = Object.assign(Object.assign({}, this.croppedImageConfig.transform), { scale: this.croppedImageConfig.scale });
        };
        FileUploadFirestorageComponent.prototype.imageCropped = function (event) {
            this.croppedImage = event;
        };
        FileUploadFirestorageComponent.prototype.imageLoaded = function () {
            // console.log('Image loaded');
        };
        FileUploadFirestorageComponent.prototype.cropperReady = function (sourceImageDimensions) {
            // console.log('Cropper ready', sourceImageDimensions);
        };
        FileUploadFirestorageComponent.prototype.loadImageFailed = function () {
            console.log('Load failed');
        };
        return FileUploadFirestorageComponent;
    }());
    FileUploadFirestorageComponent.decorators = [
        { type: i0.Component, args: [{
                    selector: 'lib-file-upload-firestorage',
                    template: "\n\n<div *ngIf=\"error.type || error.size\" class=\"error\" >\n  <span *ngIf=\"error.type\"> {{ texts.errors.type }} </span>\n  <span *ngIf=\"error.size\"> {{ texts.errors.size }} </span>\n</div>\n\n<div class=\"dropzone-wrapper\" >\n\n  <div class=\"dropzone\" dropZone (hovered)=\"toggleHover($event)\" (dropped)=\"loadTempFile($event)\"\n      [class.hovering]=\"isHovering\" *ngIf=\"!tempFile && !imageUrl\" >\n\n    <h3> {{ texts.title }} </h3>\n    <h6> {{ texts.subtitle }} </h6>\n\n    <div class=\"file\">\n      <label class=\"file-label\">\n        <input class=\"file-input\" type=\"file\" (change)=\"loadTempFile($event)\">\n        <span class=\"file-cta\">\n          <span class=\"file-icon\">\n            <i class=\"fa fa-upload\"></i>\n          </span>\n          <span class=\"file-label\">\n          {{ texts.action.or }} <strong> {{ texts.action.click }} </strong> {{ texts.action.choose }}\n          </span>\n        </span>\n      </label>\n    </div>\n  </div>\n  \n  <div *ngIf=\"tempFile && !ImageType\" style=\"flex: 1;display: flex;justify-content: center;align-items: center;\">\n      {{ tempFile.target.files[0].name }}\n  </div>\n\n  <div *ngIf=\"(tempFile && ImageType) || imageUrl\" class=\"preview-image\" >\n    <image-cropper\n        [imageChangedEvent]=\"tempFile\"\n        [imageURL]=\"imageUrl\"\n        [maintainAspectRatio]=\"cropperConfig.maintainAspectRatio\"\n        [aspectRatio]=\"cropperConfig.aspectRatio\"\n        [disabled]=\"!cropper\"\n        [canvasRotation]=\"croppedImageConfig.canvasRotation\"\n        [transform]=\"croppedImageConfig.transform\"\n        [alignImage]=\"'center'\"\n        [containWithinAspectRatio]=\"cropperConfig.containWithinAspectRatio\"\n        [resizeToWidth]=\"cropperConfig.resizeToWidth\"\n        [resizeToHeight]=\"cropperConfig.resizeToHeight\"\n        [cropperMinWidth]=\"cropperConfig.cropperMinWidth\"\n        [cropperMinHeight]=\"cropperConfig.cropperMinHeight\"\n        [cropperStaticWidth]=\"cropperConfig.cropperStaticWidth\"\n        [cropperStaticHeight]=\"cropperConfig.cropperStaticHeight\"\n        format=\"png\"\n        [autoCrop]=\"false\"\n        (imageCropped)=\"imageCropped($event)\"\n        (imageLoaded)=\"imageLoaded()\"\n        (cropperReady)=\"cropperReady($event)\"\n        (loadImageFailed)=\"loadImageFailed()\"\n    ></image-cropper>\n  </div>\n\n  <div class=\"dropzone-button-wrapper\" *ngIf=\"tempFile || imageUrl \" >\n\n    <div *ngIf=\"(ImageType && cropper) || imageUrl\" >\n      <button class=\"loader-button half-button\" (click)=\"rotateLeft()\">{{ texts.action.rotateLeft }}</button>\n      <button class=\"loader-button half-button\" (click)=\"rotateRight()\">{{ texts.action.rotateRight }}</button>\n      <hr>\n      <button class=\"loader-button half-button\" (click)=\"flipHorizontal()\">{{ texts.action.flipHorizontal }}</button>\n      <button class=\"loader-button half-button\" (click)=\"flipVertical()\"> {{ texts.action.flipVertical }}</button>\n      <hr>\n      \n      <button class=\"loader-button half-button\" (click)=\"zoomOut()\">{{ texts.action.zoomOut }}</button> \n      <button class=\"loader-button half-button\" (click)=\"zoomIn()\">{{ texts.action.zoomIn }}</button>\n      <hr>\n      \n      <button class=\"loader-button half-button\" (click)=\"resetImage()\">{{ texts.action.reset }}</button>\n      <hr>\n    </div>\n\n    <button class=\"loader-button button--success\"  (click)=\"startUpload()\" > {{ texts.button.upload }} </button>\n    <button class=\"loader-button button--warning\"  (click)=\"changeImage()\" > {{ texts.button.change }} </button>\n    \n    <div>\n\n    <div *ngIf=\"(percentage | async) as pct\">\n      <progress class=\"progress is-info\" [value]=\"pct || 0\" max=\"100\">\n      </progress>\n      {{ pct | number }}%\n    </div>\n    \n    <div *ngIf=\"(snapshot | async) as snap\">\n      {{ snap.bytesTransferred | fileSize }} of {{ snap.totalBytes | fileSize }}\n      <button (click)=\"task.pause()\" class=\"button is-warning\" [disabled]=\"!isActive(snap)\">{{ texts.button.pause }}</button>\n      <button (click)=\"task.cancel()\" class=\"button is-danger\" [disabled]=\"!isActive(snap)\">{{ texts.button.cancel }}</button>\n      <button (click)=\"task.resume()\" class=\"button is-info\" [disabled]=\"!(snap?.state === 'paused')\">{{ texts.button.resume }}</button>\n    </div>\n    \n    </div>\n  </div>\n\n</div>\n\n\n  ",
                    styles: ["\n\n    *{\n      font-family: 'sans-serif';\n    }\n\n    button:focus{\n      outline:none;\n    }\n\n    image-cropper {\n      height: 50vh;\n      min-height: 200px;\n    }\n\n    .dropzone-wrapper{\n      display:flex;\n      flex-wrap: wrap;\n\n    }\n\n    .preview-image{\n      flex: 1 0;\n      height: 100%;\n      min-height: 100px;\n      padding: .5rem;\n    }\n\n    .dropzone-button-wrapper{\n      flex: .2 0 100px;\n      box-sizing: border-box;\n      padding: .5rem 1rem;\n      box-shadow: inset 2px 0px 0px #80808047;\n    }\n\n    .dropzone {\n      flex: 1;\n      display: flex;\n      align-items: center;\n      justify-content: center;\n      flex-direction: column;\n      height: 100%;\n      min-height: 100px;\n      border: 2px dashed #f16624;\n      border-radius: 5px;\n      background: white;\n      margin: 0;\n      cursor:pointer;\n      padding: .5rem;\n      font-family: 'sans-serif';\n    }\n\n    .dropzone:hover {\n      border: 2px solid #f16624;\n      color: #dadada !important;\n    }\n\n    .dropzone .file input.file-input{\n        width: 0;\n    }\n\n    progress::-webkit-progress-value {\n        transition: width 0.1s ease;\n    }\n\n    .loader-button{\n      width: 100%;\n      padding: 0.5rem .3rem;\n      margin-bottom: 8px;\n      border: none;\n      font-size: 1rem;\n      font-family: sans-serif;\n      border-radius: .5rem;\n      cursor: pointer;\n      transition: .3s;\n    }\n\n    .half-button{\n        width: 49%;\n    }\n\n    .half-button + .half-button{\n      margin-left: 2%;\n    }\n\n    .loader-button:hover{\n      filter: brightness(0.8);\n    }\n\n    .loader-button.button--success{\n      background: #4145c7;\n      color: #ffffff;\n    }\n    \n    .loader-button.button--warning{\n      background: #ea7826;\n      color: #ffffff;\n    }\n\n    .error{\n      font-family: 'sans-serif';\n      color: #d63600;\n      font-weight: bold;\n      letter-spacing: -1px;\n    }\n\n    @media (max-width: 767px){\n      .dropzone-button-wrapper{\n        flex: 1 0 100%;\n        box-shadow: none;\n      }\n    }\n  "]
                },] }
    ];
    FileUploadFirestorageComponent.ctorParameters = function () { return [
        { type: storage.AngularFireStorage }
    ]; };
    FileUploadFirestorageComponent.propDecorators = {
        imageCropper: [{ type: i0.ViewChild, args: [ngxImageCropper.ImageCropperComponent,] }],
        res: [{ type: i0.Output }],
        lang: [{ type: i0.Input }],
        storagePath: [{ type: i0.Input }],
        maxSize: [{ type: i0.Input }],
        imageUrl: [{ type: i0.Input }],
        onlyType: [{ type: i0.Input }],
        cropper: [{ type: i0.Input }],
        cropperConfig: [{ type: i0.Input }]
    };

    var FileUploadFirestorageModule = /** @class */ (function () {
        function FileUploadFirestorageModule() {
        }
        return FileUploadFirestorageModule;
    }());
    FileUploadFirestorageModule.decorators = [
        { type: i0.NgModule, args: [{
                    declarations: [
                        FileSizePipe,
                        DropZoneDirective,
                        FileUploadFirestorageComponent,
                    ],
                    imports: [
                        storage.AngularFireStorageModule,
                        common.CommonModule,
                        ngxImageCropper.ImageCropperModule
                    ],
                    exports: [
                        FileSizePipe,
                        DropZoneDirective,
                        FileUploadFirestorageComponent,
                    ],
                    schemas: [i0.CUSTOM_ELEMENTS_SCHEMA]
                },] }
    ];

    /*
     * Public API Surface of file-upload-firestorage
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.DropZoneDirective = DropZoneDirective;
    exports.FileSizePipe = FileSizePipe;
    exports.FileUploadFirestorageComponent = FileUploadFirestorageComponent;
    exports.FileUploadFirestorageModule = FileUploadFirestorageModule;
    exports.FileUploadFirestorageService = FileUploadFirestorageService;

    Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=file-upload-firestorage.umd.js.map
