import { Pipe, EventEmitter, Directive, Output, HostListener, ɵɵdefineInjectable, Injectable, Component, ViewChild, Input, NgModule, CUSTOM_ELEMENTS_SCHEMA } from '@angular/core';
import { AngularFireStorage, AngularFireStorageModule } from '@angular/fire/storage';
import { base64ToFile, ImageCropperComponent, ImageCropperModule } from 'ngx-image-cropper';
import { CommonModule } from '@angular/common';

const FILE_SIZE_UNITS = ['B', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB', 'ZB', 'YB'];
const FILE_SIZE_UNITS_LONG = ['Bytes', 'Kilobytes', 'Megabytes', 'Gigabytes', 'Pettabytes', 'Exabytes', 'Zettabytes', 'Yottabytes'];
class FileSizePipe {
    transform(sizeInBytes, longForm) {
        const units = longForm
            ? FILE_SIZE_UNITS_LONG
            : FILE_SIZE_UNITS;
        let power = Math.round(Math.log(sizeInBytes) / Math.log(1024));
        power = Math.min(power, units.length - 1);
        const size = sizeInBytes / Math.pow(1024, power); // size in new units
        const formattedSize = Math.round(size * 100) / 100; // keep up to 2 decimals
        const unit = units[power];
        return size ? `${formattedSize} ${unit}` : '0';
    }
}
FileSizePipe.decorators = [
    { type: Pipe, args: [{
                name: 'fileSize'
            },] }
];

class DropZoneDirective {
    constructor() {
        this.dropped = new EventEmitter();
        this.hovered = new EventEmitter();
    }
    onDrop($event) {
        console.log($event);
        $event.preventDefault();
        this.dropped.emit($event.dataTransfer.files);
        this.hovered.emit(false);
    }
    onDragOver($event) {
        $event.preventDefault();
        this.hovered.emit(true);
    }
    onDragLeave($event) {
        $event.preventDefault();
        this.hovered.emit(false);
    }
}
DropZoneDirective.decorators = [
    { type: Directive, args: [{
                selector: '[dropZone]'
            },] }
];
DropZoneDirective.ctorParameters = () => [];
DropZoneDirective.propDecorators = {
    dropped: [{ type: Output }],
    hovered: [{ type: Output }],
    onDrop: [{ type: HostListener, args: ['drop', ['$event'],] }],
    onDragOver: [{ type: HostListener, args: ['dragover', ['$event'],] }],
    onDragLeave: [{ type: HostListener, args: ['dragleave', ['$event'],] }]
};

class FileUploadFirestorageService {
    constructor() { }
}
FileUploadFirestorageService.ɵprov = ɵɵdefineInjectable({ factory: function FileUploadFirestorageService_Factory() { return new FileUploadFirestorageService(); }, token: FileUploadFirestorageService, providedIn: "root" });
FileUploadFirestorageService.decorators = [
    { type: Injectable, args: [{
                providedIn: 'root'
            },] }
];
FileUploadFirestorageService.ctorParameters = () => [];

class DropZoneText {
    constructor(lang) {
        this.title = 'File Drop Zone';
        this.subtitle = 'Drop your file here';
        this.action = {
            or: 'or',
            click: 'click here',
            choose: 'to choose a file...',
            rotateLeft: 'Rotate left',
            rotateRight: 'Rotate right',
            flipHorizontal: 'Flip horizontal',
            flipVertical: 'Flip Vertical',
            zoomIn: 'Zoom +',
            zoomOut: 'Zoom -',
            reset: 'Reset image',
        };
        this.button = {
            pause: 'Pause',
            cancel: 'Cancel',
            resume: 'Resume',
            upload: 'Upload File',
            change: 'Change File'
        };
        this.errors = {
            type: 'Unsupported file type. ',
            size: 'The file is too heavy. ',
        };
        switch (lang) {
            case 'es':
                this.title = 'Area de carga';
                this.subtitle = 'Arroje su archivo acá',
                    this.action = {
                        or: 'ó',
                        click: 'Haga click aquí',
                        choose: 'para escoger su archivo...',
                        rotateLeft: 'Rotar Izq',
                        rotateRight: 'Rotar Der',
                        flipHorizontal: 'Girar horizontal',
                        flipVertical: 'Girar Vertical',
                        zoomIn: 'Acercar',
                        zoomOut: 'Alejar',
                        reset: 'Resetear',
                    };
                this.button = {
                    pause: 'Pausa',
                    cancel: 'Cancelar',
                    resume: 'Continuar',
                    upload: 'Cargar archivo',
                    change: 'Cambiar archivo'
                };
                this.errors = {
                    type: 'El tipo de archivo no es permitido.',
                    size: 'El archivo es muy grande. ',
                };
                break;
            default:
                break;
        }
    }
}

class FileUploadFirestorageComponent {
    constructor(storage) {
        this.storage = storage;
        this.res = new EventEmitter();
        this.lang = 'en';
        this.cropper = false;
        this.cropperConfig = {
            aspectRatio: 1 / 1,
            maintainAspectRatio: true,
            containWithinAspectRatio: false,
            cropperMinHeight: 0,
            cropperMinWidth: 0,
            resizeToWidth: 0,
            resizeToHeight: 0,
            cropperStaticWidth: 0,
            cropperStaticHeight: 0,
        };
        this.error = {
            type: false,
            size: false
        };
        // Cropped
        this.croppedImage = '';
        this.croppedImageConfig = {
            rotation: 0,
            canvasRotation: 0,
            showCropper: false,
            scale: 1,
            transform: {},
        };
    }
    toggleHover(event) {
        this.isHovering = event;
    }
    loadTempFile(event) {
        this.error.type = false;
        this.error.size = false;
        this.ImageType = true;
        const file = event.target ? event.target.files.item(0) : event.item(0);
        // Client-side validation example
        if (this.onlyType) {
            let allow = false;
            allow = this.onlyType.split(',').some(type => file.type.includes(type.trim()));
            if (!allow) {
                this.error.type = true;
                console.error('unsupported file type: ', file.type);
                return;
            }
        }
        if (this.maxSize && (file.size / 100000) >= this.maxSize) {
            this.error.size = true;
            console.error('unsupported size file. Its too heavy ', file.size);
            return;
        }
        if (!this.error.type && !this.error.size) {
            if (event.target) {
                this.tempFile = event;
            }
            else {
                let aux = {
                    target: {
                        files: []
                    }
                };
                aux.target.files.push(file);
                this.tempFile = aux;
            }
            if (file.type.includes('image')) {
                this.imageLoaded();
            }
            else {
                this.ImageType = false;
            }
        }
    }
    changeImage() {
        this.tempFile = null;
    }
    // startUpload(event: FileList) {
    startUpload(event) {
        let file;
        if (!this.cropper && this.tempFile) {
            file = this.tempFile.target ? this.tempFile.target.files.item(0) : this.tempFile;
        }
        else {
            file = base64ToFile(this.imageCropper.crop().base64);
        }
        // The File object
        // The storage path
        const path = `${this.storagePath}/${new Date().getTime()}_${this.tempFile ? this.tempFile.target.files[0].name : '_web'}`;
        // Totally optional metadata
        // The main task
        this.task = this.storage.upload(path, file);
        // Progress monitoring
        this.percentage = this.task.percentageChanges();
        this.snapshot = this.task.snapshotChanges();
        // The file's download URL
        this.task.then(d => {
            d.ref.getDownloadURL().then(url => {
                this.snapshot = null;
                this.percentage = null;
                this.res.emit({ thumbnail: url, locationPath: d.ref.fullPath.split('/').pop() });
            });
        });
    }
    // Determines if the upload task is active
    isActive(snapshot) {
        return snapshot.state === 'running' && snapshot.bytesTransferred < snapshot.totalBytes;
    }
    ngOnInit() {
        this.texts = new DropZoneText(this.lang);
    }
    // CropImageFunctions 
    rotateLeft() {
        this.croppedImageConfig.canvasRotation--;
        this.flipAfterRotate();
    }
    rotateRight() {
        this.croppedImageConfig.canvasRotation++;
        this.flipAfterRotate();
    }
    flipAfterRotate() {
        const flippedH = this.croppedImageConfig.transform.flipH;
        const flippedV = this.croppedImageConfig.transform.flipV;
        this.croppedImageConfig.transform = Object.assign(Object.assign({}, this.croppedImageConfig.transform), { flipH: flippedV, flipV: flippedH });
    }
    flipHorizontal() {
        this.croppedImageConfig.transform = Object.assign(Object.assign({}, this.croppedImageConfig.transform), { flipH: !this.croppedImageConfig.transform.flipH });
    }
    flipVertical() {
        this.croppedImageConfig.transform = Object.assign(Object.assign({}, this.croppedImageConfig.transform), { flipV: !this.croppedImageConfig.transform.flipV });
    }
    resetImage() {
        this.croppedImageConfig.scale = 1;
        this.croppedImageConfig.transform = {};
        this.croppedImageConfig.rotation = 0;
        this.croppedImageConfig.canvasRotation = 0;
    }
    zoomOut() {
        this.croppedImageConfig.scale -= .1;
        this.croppedImageConfig.transform = Object.assign(Object.assign({}, this.croppedImageConfig.transform), { scale: this.croppedImageConfig.scale });
    }
    zoomIn() {
        this.croppedImageConfig.scale += .1;
        this.croppedImageConfig.transform = Object.assign(Object.assign({}, this.croppedImageConfig.transform), { scale: this.croppedImageConfig.scale });
    }
    imageCropped(event) {
        this.croppedImage = event;
    }
    imageLoaded() {
        // console.log('Image loaded');
    }
    cropperReady(sourceImageDimensions) {
        // console.log('Cropper ready', sourceImageDimensions);
    }
    loadImageFailed() {
        console.log('Load failed');
    }
}
FileUploadFirestorageComponent.decorators = [
    { type: Component, args: [{
                selector: 'lib-file-upload-firestorage',
                template: `

<div *ngIf="error.type || error.size" class="error" >
  <span *ngIf="error.type"> {{ texts.errors.type }} </span>
  <span *ngIf="error.size"> {{ texts.errors.size }} </span>
</div>

<div class="dropzone-wrapper" >

  <div class="dropzone" dropZone (hovered)="toggleHover($event)" (dropped)="loadTempFile($event)"
      [class.hovering]="isHovering" *ngIf="!tempFile && !imageUrl" >

    <h3> {{ texts.title }} </h3>
    <h6> {{ texts.subtitle }} </h6>

    <div class="file">
      <label class="file-label">
        <input class="file-input" type="file" (change)="loadTempFile($event)">
        <span class="file-cta">
          <span class="file-icon">
            <i class="fa fa-upload"></i>
          </span>
          <span class="file-label">
          {{ texts.action.or }} <strong> {{ texts.action.click }} </strong> {{ texts.action.choose }}
          </span>
        </span>
      </label>
    </div>
  </div>
  
  <div *ngIf="tempFile && !ImageType" style="flex: 1;display: flex;justify-content: center;align-items: center;">
      {{ tempFile.target.files[0].name }}
  </div>

  <div *ngIf="(tempFile && ImageType) || imageUrl" class="preview-image" >
    <image-cropper
        [imageChangedEvent]="tempFile"
        [imageURL]="imageUrl"
        [maintainAspectRatio]="cropperConfig.maintainAspectRatio"
        [aspectRatio]="cropperConfig.aspectRatio"
        [disabled]="!cropper"
        [canvasRotation]="croppedImageConfig.canvasRotation"
        [transform]="croppedImageConfig.transform"
        [alignImage]="'center'"
        [containWithinAspectRatio]="cropperConfig.containWithinAspectRatio"
        [resizeToWidth]="cropperConfig.resizeToWidth"
        [resizeToHeight]="cropperConfig.resizeToHeight"
        [cropperMinWidth]="cropperConfig.cropperMinWidth"
        [cropperMinHeight]="cropperConfig.cropperMinHeight"
        [cropperStaticWidth]="cropperConfig.cropperStaticWidth"
        [cropperStaticHeight]="cropperConfig.cropperStaticHeight"
        format="png"
        [autoCrop]="false"
        (imageCropped)="imageCropped($event)"
        (imageLoaded)="imageLoaded()"
        (cropperReady)="cropperReady($event)"
        (loadImageFailed)="loadImageFailed()"
    ></image-cropper>
  </div>

  <div class="dropzone-button-wrapper" *ngIf="tempFile || imageUrl " >

    <div *ngIf="(ImageType && cropper) || imageUrl" >
      <button class="loader-button half-button" (click)="rotateLeft()">{{ texts.action.rotateLeft }}</button>
      <button class="loader-button half-button" (click)="rotateRight()">{{ texts.action.rotateRight }}</button>
      <hr>
      <button class="loader-button half-button" (click)="flipHorizontal()">{{ texts.action.flipHorizontal }}</button>
      <button class="loader-button half-button" (click)="flipVertical()"> {{ texts.action.flipVertical }}</button>
      <hr>
      
      <button class="loader-button half-button" (click)="zoomOut()">{{ texts.action.zoomOut }}</button> 
      <button class="loader-button half-button" (click)="zoomIn()">{{ texts.action.zoomIn }}</button>
      <hr>
      
      <button class="loader-button half-button" (click)="resetImage()">{{ texts.action.reset }}</button>
      <hr>
    </div>

    <button class="loader-button button--success"  (click)="startUpload()" > {{ texts.button.upload }} </button>
    <button class="loader-button button--warning"  (click)="changeImage()" > {{ texts.button.change }} </button>
    
    <div>

    <div *ngIf="(percentage | async) as pct">
      <progress class="progress is-info" [value]="pct || 0" max="100">
      </progress>
      {{ pct | number }}%
    </div>
    
    <div *ngIf="(snapshot | async) as snap">
      {{ snap.bytesTransferred | fileSize }} of {{ snap.totalBytes | fileSize }}
      <button (click)="task.pause()" class="button is-warning" [disabled]="!isActive(snap)">{{ texts.button.pause }}</button>
      <button (click)="task.cancel()" class="button is-danger" [disabled]="!isActive(snap)">{{ texts.button.cancel }}</button>
      <button (click)="task.resume()" class="button is-info" [disabled]="!(snap?.state === 'paused')">{{ texts.button.resume }}</button>
    </div>
    
    </div>
  </div>

</div>


  `,
                styles: [`

    *{
      font-family: 'sans-serif';
    }

    button:focus{
      outline:none;
    }

    image-cropper {
      height: 50vh;
      min-height: 200px;
    }

    .dropzone-wrapper{
      display:flex;
      flex-wrap: wrap;

    }

    .preview-image{
      flex: 1 0;
      height: 100%;
      min-height: 100px;
      padding: .5rem;
    }

    .dropzone-button-wrapper{
      flex: .2 0 100px;
      box-sizing: border-box;
      padding: .5rem 1rem;
      box-shadow: inset 2px 0px 0px #80808047;
    }

    .dropzone {
      flex: 1;
      display: flex;
      align-items: center;
      justify-content: center;
      flex-direction: column;
      height: 100%;
      min-height: 100px;
      border: 2px dashed #f16624;
      border-radius: 5px;
      background: white;
      margin: 0;
      cursor:pointer;
      padding: .5rem;
      font-family: 'sans-serif';
    }

    .dropzone:hover {
      border: 2px solid #f16624;
      color: #dadada !important;
    }

    .dropzone .file input.file-input{
        width: 0;
    }

    progress::-webkit-progress-value {
        transition: width 0.1s ease;
    }

    .loader-button{
      width: 100%;
      padding: 0.5rem .3rem;
      margin-bottom: 8px;
      border: none;
      font-size: 1rem;
      font-family: sans-serif;
      border-radius: .5rem;
      cursor: pointer;
      transition: .3s;
    }

    .half-button{
        width: 49%;
    }

    .half-button + .half-button{
      margin-left: 2%;
    }

    .loader-button:hover{
      filter: brightness(0.8);
    }

    .loader-button.button--success{
      background: #4145c7;
      color: #ffffff;
    }
    
    .loader-button.button--warning{
      background: #ea7826;
      color: #ffffff;
    }

    .error{
      font-family: 'sans-serif';
      color: #d63600;
      font-weight: bold;
      letter-spacing: -1px;
    }

    @media (max-width: 767px){
      .dropzone-button-wrapper{
        flex: 1 0 100%;
        box-shadow: none;
      }
    }
  `]
            },] }
];
FileUploadFirestorageComponent.ctorParameters = () => [
    { type: AngularFireStorage }
];
FileUploadFirestorageComponent.propDecorators = {
    imageCropper: [{ type: ViewChild, args: [ImageCropperComponent,] }],
    res: [{ type: Output }],
    lang: [{ type: Input }],
    storagePath: [{ type: Input }],
    maxSize: [{ type: Input }],
    imageUrl: [{ type: Input }],
    onlyType: [{ type: Input }],
    cropper: [{ type: Input }],
    cropperConfig: [{ type: Input }]
};

class FileUploadFirestorageModule {
}
FileUploadFirestorageModule.decorators = [
    { type: NgModule, args: [{
                declarations: [
                    FileSizePipe,
                    DropZoneDirective,
                    FileUploadFirestorageComponent,
                ],
                imports: [
                    AngularFireStorageModule,
                    CommonModule,
                    ImageCropperModule
                ],
                exports: [
                    FileSizePipe,
                    DropZoneDirective,
                    FileUploadFirestorageComponent,
                ],
                schemas: [CUSTOM_ELEMENTS_SCHEMA]
            },] }
];

/*
 * Public API Surface of file-upload-firestorage
 */

/**
 * Generated bundle index. Do not edit.
 */

export { DropZoneDirective, FileSizePipe, FileUploadFirestorageComponent, FileUploadFirestorageModule, FileUploadFirestorageService };
//# sourceMappingURL=file-upload-firestorage.js.map
